from django.contrib.auth import password_validation
from django.core.exceptions import ValidationError
from django.forms import ModelForm, EmailField, CharField, EmailInput, TextInput, PasswordInput
from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm, ReadOnlyPasswordHashField
from django.utils.translation import gettext_lazy as _

from .models import User


class UserCreationForm(ModelForm):
    error_messages = {
        "password_mismatch": _("The two password fields didn’t match."),
    }
    # email = EmailField(
    #     label=_("Email Address"),
    #     widget=EmailInput(attrs={'class': 'input-field', 'placeholder': _('Enter your email'), 'required': True}),
    # )
    # username = CharField(
    #     label=_("Username"),
    #     widget=TextInput(attrs={'class': 'input-field', 'placeholder': _('Choose a username'), 'required': True}),
    # )
    password1 = CharField(
        label=_("Password"),
        strip=False,
        widget=PasswordInput(
            attrs={
                'class': 'input-field',
                'placeholder': _('Enter your password'),
                'required': True,
                'autocomplete': 'new-password'
            }
        ),
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = CharField(
        label=_("Password confirmation"),
        widget=PasswordInput(
            attrs={
                'class': 'input-field',
                'placeholder': _('Confirm your password'),
                'required': True,
                'autocomplete': 'new-password'
            }
        ),
        strip=False,
        help_text=_("Enter the same password as before, for verification."),
    )

    class Meta:
        model = User
        fields = ('email', 'username')

    def clean_password2(self):
        password1 = self.cleaned_data.get("password")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise ValidationError(
                self.error_messages["password_mismatch"],
                code="password_mismatch",
            )
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class UserChangeForm(ModelForm):
    password = ReadOnlyPasswordHashField(
        label=_("Password"),
        help_text=_(
            "Raw passwords are not stored, so there is no way to see this "
            "user’s password, but you can change the password using "
            '<a href="{}">this form</a>.'
        ),
    )

    class Meta:
        model = User
        fields = ("email", "password", "is_active")
